@@ -7,11 +7,12 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from typing import cast, Any, Dict, Iterator, List, MutableMapping, Optional, Union
+from typing import cast, Any, Iterator, List, Optional, Union
 
+from .namespaces import NamespacesType
 from .exceptions import ElementPathTypeError
 from .protocols import ElementProtocol, LxmlElementProtocol, \
-    DocumentProtocol, XsdElementProtocol
+    DocumentProtocol, LxmlDocumentProtocol, XsdElementProtocol
 from .etree import is_etree_document, is_etree_element
 from .xpath_nodes import SchemaElemType, ChildNodeType, ElementMapType, \
     TextNode, CommentNode, ProcessingInstructionNode, \
@@ -23,52 +24,76 @@ __all__ = ['RootArgType', 'get_node_tree', 'build_node_tree',
 RootArgType = Union[DocumentProtocol, ElementProtocol, SchemaElemType,
                     'DocumentNode', 'ElementNode']
 
+ElementTreeRootType = Union[DocumentProtocol, ElementProtocol]
+LxmlRootType = Union[LxmlDocumentProtocol, LxmlElementProtocol]
+
 
 def is_schema(obj: Any) -> bool:
     return hasattr(obj, 'xsd_version') and hasattr(obj, 'maps') and not hasattr(obj, 'parent')
 
 
-def get_node_tree(root: RootArgType, namespaces: Optional[Dict[str, str]] = None) \
-        -> Union[DocumentNode, ElementNode]:
+def get_node_tree(root: RootArgType,
+                  namespaces: Optional[NamespacesType] = None,
+                  uri: Optional[str] = None,
+                  fragment: bool = False) -> Union[DocumentNode, ElementNode]:
     """
     Returns a tree of XPath nodes that wrap the provided root tree.
 
     :param root: an Element or an ElementTree or a schema or a schema element.
     :param namespaces: an optional mapping from prefixes to namespace URIs, \
     Ignored if root is a lxml etree or a schema structure.
+    :param uri: an optional URI associated with the root element or the document.
+    :param fragment: if `True` a root element is considered a fragment, otherwise \
+    a root element is considered the root of an XML document. If the root is a \
+    document node or an ElementTree instance, and fragment is `True` then use the \
+    root element and returns an element node.
     """
     if isinstance(root, (DocumentNode, ElementNode)):
+        if uri is not None and root.uri is None:
+            root.uri = uri
+
+        if fragment and isinstance(root, DocumentNode):
+            root_node = root.getroot()
+            if root_node.uri is None:
+                root_node.uri = root.uri
+            return root_node
+
         return root
-    elif is_etree_document(root):
-        if hasattr(root, 'xpath'):
-            return build_lxml_node_tree(cast(DocumentProtocol, root))
-        return build_node_tree(
-            cast(DocumentProtocol, root), namespaces
-        )
-    elif hasattr(root, 'xsd_version') and hasattr(root, 'maps'):
-        # schema or schema node
-        return build_schema_node_tree(
-            cast(SchemaElemType, root)
-        )
-    elif is_etree_element(root) and not callable(root.tag):  # type: ignore[union-attr]
-        if hasattr(root, 'nsmap') and hasattr(root, 'xpath'):
-            return build_lxml_node_tree(cast(LxmlElementProtocol, root))
-        return build_node_tree(
-            cast(ElementProtocol, root), namespaces
-        )
+
+    # If a fragment is requested remove the ElementTree instance
+    if is_etree_document(root):
+        _root = cast(DocumentProtocol, root).getroot() if fragment else root
+    elif is_etree_element(root) and not callable(cast(ElementProtocol, root).tag):
+        _root = root
     else:
         msg = "invalid root {!r}, an Element or an ElementTree or a schema node required"
         raise ElementPathTypeError(msg.format(root))
 
+    if hasattr(_root, 'xpath'):
+        # a lxml element tree data
+        return build_lxml_node_tree(
+            cast(LxmlRootType, _root), uri, fragment
+        )
+    elif hasattr(_root, 'xsd_version') and hasattr(_root, 'maps'):
+        # a schema or a schema node
+        return build_schema_node_tree(
+            cast(SchemaElemType, root), uri
+        )
+    else:
+        return build_node_tree(
+            cast(ElementTreeRootType, root), namespaces, uri
+        )
 
-def build_node_tree(root: Union[DocumentProtocol, ElementProtocol],
-                    namespaces: Optional[MutableMapping[str, str]] = None) \
-        -> Union[DocumentNode, ElementNode]:
+
+def build_node_tree(root: ElementTreeRootType,
+                    namespaces: Optional[NamespacesType] = None,
+                    uri: Optional[str] = None) -> Union[DocumentNode, ElementNode]:
     """
     Returns a tree of XPath nodes that wrap the provided root tree.
 
     :param root: an Element or an ElementTree.
     :param namespaces: an optional mapping from prefixes to namespace URIs.
+    :param uri: an optional URI associated with the document or the root element.
     """
     root_node: Union[DocumentNode, ElementNode]
     parent: Any
@@ -96,11 +121,16 @@ def build_node_tree(root: Union[DocumentProtocol, ElementProtocol],
 
     if hasattr(root, 'parse'):
         document = cast(DocumentProtocol, root)
-        root_node = parent = DocumentNode(document, position)
+        parent = DocumentNode(document, uri, position)
+        root_node = parent
         position += 1
         elements = root_node.elements
 
-        elem = document.getroot()
+        root_elem = document.getroot()
+        if root_elem is None:
+            return root_node
+
+        elem = root_elem
         child = build_element_node()
         parent.children.append(child)
         parent = child
@@ -110,6 +140,8 @@ def build_node_tree(root: Union[DocumentProtocol, ElementProtocol],
         elements = {}
         root_node = parent = build_element_node()
         root_node.elements = elements
+        if uri is not None:
+            root_node.uri = uri
 
     children = iter(elem)
     iterators: List[Any] = []
@@ -143,12 +175,16 @@ def build_node_tree(root: Union[DocumentProtocol, ElementProtocol],
                 return root_node
 
 
-def build_lxml_node_tree(root: Union[DocumentProtocol, LxmlElementProtocol]) \
-        -> Union[DocumentNode, ElementNode]:
+def build_lxml_node_tree(root: LxmlRootType,
+                         uri: Optional[str] = None,
+                         fragment: bool = False) -> Union[DocumentNode, ElementNode]:
     """
     Returns a tree of XPath nodes that wrap the provided lxml root tree.
 
     :param root: a lxml Element or a lxml ElementTree.
+    :param uri: an optional URI associated with the document or the root element.
+    :param fragment: if `True` a root element is considered a fragment, otherwise \
+    a root element is considered the root of an XML document.
     """
     root_node: Union[DocumentNode, ElementNode]
     parent: Any
@@ -198,24 +234,43 @@ def build_lxml_node_tree(root: Union[DocumentProtocol, LxmlElementProtocol]) \
 
         return node
 
-    if hasattr(root, 'parse'):
-        document = cast(DocumentProtocol, root)
-        root_node = parent = DocumentNode(document, position)
+    if fragment:
+        if hasattr(root, 'parse'):
+            root_elem = cast(LxmlDocumentProtocol, root).getroot()
+            if root_elem is None:
+                msg = "requested a fragment of an empty ElementTree document"
+                raise ElementPathTypeError(msg)
+            elem = root_elem
+        else:
+            elem = root
+
+        parent = None
+        elements = {}
+        root_node = parent = build_lxml_element_node()
+        root_node.elements = elements
+
+    elif hasattr(root, 'parse'):
+        document = cast(LxmlDocumentProtocol, root)
+        root_node = parent = DocumentNode(document, position=position)
         position += 1
 
-        elem = cast(LxmlElementProtocol, document.getroot())
-        if elem is None:
+        root_elem = document.getroot()
+        if root_elem is None:
             return root_node
 
+        elem = root_elem
         elements = root_node.elements
         parent = build_document_node()
 
     elif root.getparent() is None:
-        # if it's the effective root of the tree creates a root
-        # document node with none value and position==0
+        # Creates a root document node for parsing root's siblings.
         document = root.getroottree()
-        root_node = parent = DocumentNode(document, 0)
-        elem = root
+        root_node = parent = DocumentNode(document, position=0)
+
+        root_elem = document.getroot()
+        assert root_elem is not None
+
+        elem = root_elem
         elements = root_node.elements
         parent = build_document_node()
 
@@ -224,7 +279,6 @@ def build_lxml_node_tree(root: Union[DocumentProtocol, LxmlElementProtocol]) \
             parent.elements = root_node.elements
             parent.parent = None
             root_node = parent
-
     else:
         elem = root
         parent = None
@@ -236,6 +290,9 @@ def build_lxml_node_tree(root: Union[DocumentProtocol, LxmlElementProtocol]) \
     iterators: List[Any] = []
     ancestors: List[Any] = []
 
+    if uri is not None:
+        root_node.uri = uri
+
     while True:
         for elem in children:
             if not callable(elem.tag):
@@ -265,6 +322,7 @@ def build_lxml_node_tree(root: Union[DocumentProtocol, LxmlElementProtocol]) \
 
 
 def build_schema_node_tree(root: SchemaElemType,
+                           uri: Optional[str] = None,
                            elements: Optional[ElementMapType] = None,
                            global_elements: Optional[List[ChildNodeType]] = None) \
         -> SchemaElementNode:
@@ -272,6 +330,7 @@ def build_schema_node_tree(root: SchemaElemType,
     Returns a tree of XPath nodes that wrap the provided XSD schema structure.
 
     :param root: a schema or a schema element.
+    :param uri: an optional URI associated with the root element.
     :param elements: a shared map from XSD elements to tree nodes. Provided for \
     linking together parts of the same schema or other schemas.
     :param global_elements: a list for schema global elements, used for linking \
@@ -302,6 +361,8 @@ def build_schema_node_tree(root: SchemaElemType,
     parent = None
     root_node = parent = build_schema_element_node()
     root_node.elements = _elements
+    if uri is not None:
+        root_node.uri = uri
 
     if global_elements is not None:
         global_elements.append(root_node)
@@ -354,7 +415,7 @@ def build_schema_node_tree(root: SchemaElemType,
                     else:
                         # Extend node tree with other globals
                         element_node.ref = build_schema_node_tree(
-                            ref, _elements, global_elements
+                            ref, elements=_elements, global_elements=global_elements
                         )
 
                 return root_node
