@@ -218,7 +218,7 @@ class _PrefixedReferenceToken(XPathToken):
             name = '{%s}%s' % (self.get_namespace(self[0].value), self[1].value)
 
         if context is None:
-            yield name
+            raise self.missing_context()
         elif isinstance(context, XPathSchemaContext):
             yield from self.select_xsd_nodes(context, name)
 
@@ -720,12 +720,12 @@ def select_child_path(self, context=None):
         if isinstance(context.root, DocumentNode):
             yield context.root
     elif len(self) == 1:
-        if context.item is context.root and context.item.parent is not None:
-            return  # A rooted subtree -> document root produce []
-        elif not isinstance(context, XPathSchemaContext):
-            context.item = None
+        if isinstance(context.document, DocumentNode):
+            context.item = context.document
+        elif context.root is None or isinstance(context.root.parent, ElementNode):
+            return  # No root or a rooted subtree -> document root produce []
         else:
-            context.item = context.root
+            context.item = context.root  # A fragment or a schema node
         yield from self[0].select(context)
     else:
         items = set()
@@ -778,12 +778,12 @@ def select_descendant_path(self, context=None):
                             self[1].add_xsd_type(result)
 
     else:
-        if context.item is context.root and context.item.parent is not None:
-            return  # A rooted subtree -> document root produce []
-        elif not isinstance(context, XPathSchemaContext):
-            context.item = None
+        if isinstance(context.document, DocumentNode):
+            context.item = context.document
+        elif context.root is None or isinstance(context.root.parent, ElementNode):
+            return  # No root or a rooted subtree -> document root produce []
         else:
-            context.item = context.root
+            context.item = context.root  # A fragment or a schema node
 
         items = set()
         for _ in context.iter_descendants():
